unit ListExpT;
{
TListExpert is placed in the public domain by Guy Smith (Capella Software Ltd).
You may freely copy this expert. You may modify this expert providing you leave
this original notice intact and you make your modifications apparent.

Use this software at your own risk. Neither Guy Smith nor Capella Software Ltd.
accept any responsibility for any loss or damages which result from using this
software.

-------------------------------------------------------------------------------

TListExpert is an expert which generates code for a TList subclass.
The purpose of the TList subclass is to provide a homogenous list of
objects of a specific class. For example a TList of TAddress objects.


}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExptIntf;

type
  TListExpert = class (TIExpert)
  public
    function GetStyle: TExpertStyle; override;
    function GETIDString: string; override;
    function GetName: string; override;
    function GetMenuText: string; override;
    function GetState: TExpertState; override;
    function GetAuthor: string; override;
    function GetGlyph: HICON; override;
    function GetPage: string; override;
    function GetComment: string; override;
    procedure Execute; override;
  end;

  TListParams = class
  public
    FileName: string;
    ObjectName: string;
    ClassName: string;
    PluralObjectName: string;
    ListClassName: string;
    FindFunctionPointerName: string;
    ClassDefinition: TStringList;
    CustomTemplate: string;
  end;

  TfrmListExpert = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    edtFileName: TEdit;
    edtObjectName: TEdit;
    edtClassName: TEdit;
    edtPluralObjectName: TEdit;
    edtListClassName: TEdit;
    edtFindFunctionPointerName: TEdit;
    mmoClassDefinition: TMemo;
    Memo2: TMemo;
    btnFinish: TButton;
    btnCancel: TButton;
    btnClearFields: TButton;
    SaveDialog: TSaveDialog;
    bbnFileName: TBitBtn;
    GroupBox1: TGroupBox;
    rboDefault: TRadioButton;
    rboCustom: TRadioButton;
    edtCustom: TEdit;
    bbnCustom: TBitBtn;
    CustomSaveDialog: TSaveDialog;
    procedure edtFileNameChange(Sender: TObject);
    procedure edtObjectNameChange(Sender: TObject);
    procedure edtClassNameChange(Sender: TObject);
    procedure edtPluralObjectNameChange(Sender: TObject);
    procedure edtListClassNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure edtFileNameExit(Sender: TObject);
    procedure btnClearFieldsClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
    procedure bbnFileNameClick(Sender: TObject);
    procedure rboCustomClick(Sender: TObject);
    procedure rboDefaultClick(Sender: TObject);
    procedure bbnCustomClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    blnFileNameChanged: boolean;
    blnObjectNameChanged: boolean;
    blnClassNameChanged: boolean;
    blnPluralObjectNameChanged: boolean;
    blnListClassNameChanged: boolean;

    procedure UpdateObjectName;
    procedure UpdateClassName;
    procedure UpdateListClassName;
    procedure UpdateFindFunctionPointerName;
    procedure UpdatePluralObjectName;
  end;

  function ListTranslateStream(ListParams: TListParams): TMemoryStream;

  function GetDefaultListTemplateStream: TMemoryStream;

  procedure Register;

var
  frmListExpert: TfrmListExpert;

implementation

const
  CRLF = #13+#10;

{$R *.DFM}

function TStringsToString(stlInput: TStrings; strIndent: string): string;
// convert a TStrings to a single concatenated string
var
  strOutput: string;
  intString: integer;
begin
  for intString:=0 to stlInput.Count - 1 do
    if intString=0 then
      strOutput:=strOutput+stlInput.Strings[intString]+#13+#10
    else
      strOutput:=strOutput+strIndent+stlInput.Strings[intString]+#13+#10;

  Result:=strOutput;
end;

function TranslateCandidate(
strStart: string; strCandidate: string; ListParams: TListParams; strEnd: string): string;
begin
  Result:=strStart+strCandidate+strEnd;
  strCandidate:=Uppercase(strCandidate);
  if strCandidate='FILENAME' then
    Result:=ListParams.FileName
  else if strCandidate='UNITNAME' then
  begin
    Result:=ExtractFileName(ListParams.FileName);
    if Length(ExtractFileExt(Result)) > 0 then
      Result:=Copy(Result, 1, Length(Result) - Length(ExtractFileExt(Result)));
  end
  else if strCandidate='OBJECTNAME' then
    Result:=ListParams.ObjectName
  else if strCandidate='CLASSNAME' then
    Result:=ListParams.ClassName
  else if strCandidate='PLURALOBJECTNAME' then
    Result:=ListParams.PluralObjectName
  else if strCandidate='LISTCLASSNAME' then
    Result:=ListParams.ListClassName
  else if strCandidate='FINDFUNCTIONPOINTERNAME' then
    Result:=ListParams.FindFunctionPointerName
  else if strCandidate='CLASSDEFINITION' then
    Result:=TStringsToString(ListParams.ClassDefinition, '  ')
  else if strCandidate='CUSTOMTEMPLATE' then
    Result:=ListParams.CustomTemplate;
end;

function ListTranslateStream(ListParams: TListParams): TMemoryStream;
var
  stmTemplate : TStream;
  stmOutput   : TMemoryStream;
  chr         : char;
  chr2        : char;
  chr3        : char;
  chr4        : char;
  strCandidate: string;
  blnWriteChar: boolean;
begin
  try
    if (Length(ListParams.CustomTemplate) > 0) and
    FileExists(ListParams.CustomTemplate) then
      stmTemplate:=
      TFileStream.Create(ListParams.CustomTemplate, fmOpenRead or fmShareDenyWrite)
    else
      stmTemplate:=GetDefaultListTemplateStream;


    stmTemplate.Position:=0;
    stmOutput  :=TMemoryStream.Create;

    while stmTemplate.Position < stmTemplate.Size do
    begin
      stmTemplate.ReadBuffer(chr, 1);

      blnWriteChar:=True;
      if (chr='<') and (stmTemplate.Position+1 < stmTemplate.Size) then
      begin
        stmTemplate.ReadBuffer(chr2, 1);
        if chr2='<' then
        begin
          // this is a candidate for conversion
          strCandidate:='';
          while stmTemplate.Position < stmTemplate.Size do
          begin
            stmTemplate.ReadBuffer(chr3, 1);
            if (chr3='>') or (chr3=#13) then
            begin
              stmTemplate.ReadBuffer(chr4, 1);
              if ((chr3='>') and (chr4='>')) or ((chr3=#13) and (chr4=#10)) then
              begin
                strCandidate:=
                TranslateCandidate(chr+chr2, strCandidate, ListParams, chr3+chr4);

                stmOutput.WriteBuffer(strCandidate[1], Length(strCandidate));
                blnWriteChar:=False;
                Break;
              end;
            end;
            strCandidate:=strCandidate+chr3;
          end;

        end
        else
        begin
          stmOutput.WriteBuffer(chr, 1);
          chr:=chr2;
        end;
      end;

      if blnWriteChar then
        stmOutput.WriteBuffer(chr, 1);

    end;
    stmOutput.Position:=0;

  finally
    stmTemplate.Free;
  end;
  Result:=stmOutput;
end;

procedure TfrmListExpert.edtFileNameChange(Sender: TObject);
begin
  blnFileNameChanged:=True;
end;

procedure TfrmListExpert.edtObjectNameChange(Sender: TObject);
begin
  blnObjectNameChanged:=True;
end;

procedure TfrmListExpert.edtClassNameChange(Sender: TObject);
begin
  blnClassNameChanged:=True;
end;

procedure TfrmListExpert.edtPluralObjectNameChange(Sender: TObject);
begin
  blnPluralObjectNameChanged:=True;
end;

procedure TfrmListExpert.edtListClassNameChange(Sender: TObject);
begin
  blnListClassNameChanged:=True;
end;

procedure TfrmListExpert.FormCreate(Sender: TObject);
begin
  blnFileNameChanged:=False;
  blnObjectNameChanged:=False;
  blnClassNameChanged:=False;
  blnPluralObjectNameChanged:=False;
  blnListClassNameChanged:=False;
end;

procedure TfrmListExpert.edtFileNameExit(Sender: TObject);
begin
  if not (Length(edtFileName.Text) = 0) then
    UpdateObjectName;
end;

procedure TfrmListExpert.UpdateObjectName;
begin
  if (Length(edtObjectName.Text)=0) or not blnObjectNameChanged then
  begin
    edtObjectName.Text:=ExtractFileName(edtFileName.Text);
    if Length(ExtractFileExt(edtObjectName.Text)) > 0 then
      edtObjectName.Text:=Copy(edtObjectName.Text, 1,
      Length(edtObjectName.Text) - Length(ExtractFileExt(edtObjectName.Text)));

    UpdateClassName;
  end;
end;

procedure TfrmListExpert.UpdateClassName;
begin
  if (Length(edtClassName.Text)=0) or not blnClassNameChanged then
  begin
    edtClassName.Text:='T'+edtObjectName.Text;
    UpdatePluralObjectName;
    UpdateListClassName;
  end;
end;

procedure TfrmListExpert.UpdateListClassName;
begin
  if (Length(edtListClassName.Text)=0) or not blnListClassNameChanged then
  begin
    edtListClassName.Text:=edtClassName.Text+'List';
    UpdateFindFunctionPointerName;
  end;
end;

procedure TfrmListExpert.UpdateFindFunctionPointerName;
begin
  if (Length(edtFindFunctionPointerName.Text)=0) then
  begin
    edtFindFunctionPointerName.Text:=edtListClassName.Text+'Find';
  end;
end;

procedure TfrmListExpert.UpdatePluralObjectName;
begin
  if (Length(edtPluralObjectName.Text)=0) or not blnPluralObjectNameChanged then
  begin
    if (Length(edtObjectName.Text) > 0) and
    (edtObjectName.Text[Length(edtObjectName.Text)]='s') then
      edtPluralObjectName.Text:=edtObjectName.Text+'es'
    else if (Length(edtObjectName.Text) > 1) and
    (Copy(edtObjectName.Text, Length(edtObjectName.Text) - 1, 2)='ey') then
      edtPluralObjectName.Text:=Copy(edtObjectName.Text, 1, Length(edtObjectName.Text) - 2)+'ies'
    else
      edtPluralObjectName.Text:=edtObjectName.Text+'s';
  end;
end;

procedure TfrmListExpert.btnClearFieldsClick(Sender: TObject);
begin
  edtFileName.Text:='';
  edtObjectName.Text:='';
  edtClassName.Text:='';
  edtPluralObjectName.Text:='';
  edtListClassName.Text:='';
  edtFindFunctionPointerName.Text:='';
  mmoClassDefinition.Clear;
end;

procedure TfrmListExpert.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmListExpert.btnFinishClick(Sender: TObject);
var
  ListParams: TListParams;
  Stream: TStream;
  FileStream: TFileStream;
  chr: Char;
begin
  if Length(edtFileName.Text) = 0 then
    MessageDlg('Must enter a filename', mtError, [mbOk], 0)
  else if (rboCustom.Checked) and (Length(edtCustom.Text) = 0) then
    MessageDlg('Must enter a custom code template filename', mtError, [mbOk], 0)
  else if (rboCustom.Checked) and not FileExists(edtCustom.Text) then
    MessageDlg('Custom code template file does not exist', mtError, [mbOk], 0)
  else
  begin
    if Length(ExtractFileExt(edtFileName.Text))=0 then
      edtFileName.Text:=edtFileName.Text+'.PAS';

    if (not FileExists(edtFileName.Text)) or (MessageDlg(edtfileName.Text+' file exists'+
    #13+#10+'Overwrite ?', mtConfirmation, [mbYes, mbNo], 0)=mrYes) then
    begin

      ListParams:=TListParams.Create;
      ListParams.FileName:=edtFileName.Text;
      ListParams.ObjectName:=edtObjectName.Text;
      ListParams.ClassName:=edtClassName.Text;
      ListParams.PluralObjectName:=edtPluralObjectName.Text;
      ListParams.ListClassName:=edtListClassName.Text;
      ListParams.FindFunctionPointerName:=edtFindFunctionPointerName.Text;
      ListParams.ClassDefinition:=TStringList(mmoClassDefinition.Lines);

      if rboCustom.Checked then
        ListParams.CustomTemplate:=edtCustom.Text;

      Stream:=ListTranslateStream(ListParams);

      try
        FileStream:=TFileStream.Create(edtFileName.Text, fmCreate);
        Stream.Position:=0;

        while Stream.Position < Stream.Size do
        begin
          Stream.ReadBuffer(chr, 1);
          FileStream.WriteBuffer(chr, 1);
        end;
      finally
        FileStream.Free;
      end;

      MessageDlg('List Class written to '+edtFileName.Text, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TfrmListExpert.bbnFileNameClick(Sender: TObject);
begin
  SaveDialog.FileName:=edtfileName.Text;
  if Savedialog.Execute then edtFileName.Text:=SaveDialog.FileName;
end;

function GetDefaultListTemplateStream: TMemoryStream;
var
  Stream: TMemoryStream;
  strBuffer: string;
begin
  Stream:=TMemoryStream.Create;

strBuffer:='unit <<UnitName>>;'+CRLF+
CRLF+
'interface'+CRLF+
CRLF+
'uses'+CRLF+
'  Classes;'+CRLF+
CRLF+
'type'+CRLF+
'  <<ClassDefinition>>'+CRLF+
CRLF+
'  <<FindFunctionPointerName>> = function (<<ObjectName>>: <<ClassName>>): boolean;'+CRLF+
CRLF+
'  <<ListClassName>> = class(TList)'+CRLF+
'  protected'+CRLF+
'    function  Get<<ObjectName>>(Index: integer): <<ClassName>>;';

strBuffer:=strBuffer+
CRLF+
'    procedure Put<<ObjectName>>(Index: integer; <<ObjectName>>: <<ClassName>>);'+CRLF+
'  public'+CRLF+
'    property <<PluralObjectName>>[Index: integer]: <<ClassName>> read Get<<ObjectName>> write Put<<ObjectName>>;'+CRLF+
'    destructor Destroy; override;'+CRLF+
'    function Find(Find: <<FindFunctionPointerName>>; var Found: integer):boolean;'+CRLF+
'  end;'+CRLF+
CRLF+
'implementation'+CRLF+
CRLF+
'function <<ListClassName>>.Get<<ObjectName>>(Index: integer): <<ClassName>>;'+CRLF+
'begin'+CRLF+
'  Result:=<<ClassName>>(Items[Index]);'+CRLF+
'end;'+CRLF+
CRLF+
'procedure <<ListClassName>>.Put<<ObjectName>>(Index: integer; <<ObjectName>>: <<ClassName>>);'+CRLF+
'begin'+CRLF+
'  Items[Index]:=<<ObjectName>>;'+CRLF+
'end;'+CRLF+
CRLF+
'destructor <<ListClassName>>.Destroy;'+CRLF+
'var'+CRLF+
'  Index: integer;'+CRLF+
'begin'+CRLF+
'  // free all of the objects first'+CRLF+
'  for Index:=0 to Count - 1 do'+CRLF+
'    <<ClassName>>(Items[Index]).Free;'+CRLF+
CRLF+
'  inherited;'+CRLF+
'end;'+CRLF+
CRLF+
'function <<ListClassName>>.Find(Find: <<ListClassName>>Find; var Found: integer):boolean;'+CRLF+
'var'+CRLF+
'  Index: integer;'+CRLF+
'begin'+CRLF+
'  Result:=False;'+CRLF+
'  Found:=-1;'+CRLF+
'  for Index:=0 to Count - 1 do'+CRLF+
'    if Find(<<ClassName>>(Items[Index])) then'+CRLF+
'    begin'+CRLF+
'      Result:=True;'+CRLF+
'      Found:=Index;'+CRLF+
'      Break;'+CRLF+
'    end;'+CRLF+
'end;'+CRLF+
CRLF+
'end.'+CRLF;

  Stream.WriteBuffer(strBuffer[1], Length(strBuffer));
  Stream.Position:=0;
  Result:=Stream;
end;

function TListExpert.GetStyle: TExpertStyle;
begin
  Result:=esStandard;
end;

function TListExpert.GETIDString: string;
begin
  Result:='CA.TListExpert';
end;

function TListExpert.GetName: string;
begin
  Result:='TListExpert';
end;

function TListExpert.GetMenuText: string;
begin
  Result:='&List Expert...';
end;

function TListExpert.GetState: TExpertState;
begin
  Result:=[esEnabled];
end;

function TListExpert.GetAuthor: string;
begin
  Result:='Guy Smith';
end;

function TListExpert.GetGlyph: HICON;
begin
  Result:=0;
end;

function TListExpert.GetPage: string;
begin
  Result:='';
end;

function TListExpert.GetComment: string;
begin
  Result:='';
end;

procedure TListExpert.Execute;
var
  frmListExpert: TfrmListExpert;
begin
  frmListExpert:=TfrmListExpert.Create(Application);
  frmListExpert.Show;
end;

procedure Register;
begin
  RegisterLibraryExpert(TListExpert.Create);
end;

procedure TfrmListExpert.rboCustomClick(Sender: TObject);
begin
  edtCustom.Enabled:=True;
  edtCustom.SetFocus;
end;

procedure TfrmListExpert.rboDefaultClick(Sender: TObject);
begin
  edtCustom.Enabled:=False;
end;

procedure TfrmListExpert.bbnCustomClick(Sender: TObject);
begin
  CustomSaveDialog.FileName:=edtCustom.Text;
  if CustomSaveDialog.Execute then edtCustom.Text:=CustomSaveDialog.FileName;
end;

end.
